package com.yunan.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.yunan.constant.ResponseCode;
import com.yunan.dto.ApiResponse;
import com.yunan.dto.RegisterDTO;
import com.yunan.mapper.AuthMapper;
import com.yunan.pojo.User;
import com.yunan.service.AuthService;
import com.yunan.util.VerificationCodeUtils;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.mail.SimpleMailMessage;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.PathVariable;

import javax.annotation.Resource;
import javax.mail.internet.AddressException;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeUtility;
import java.io.UnsupportedEncodingException;
import java.util.concurrent.TimeUnit;


@Service
public class AuthServiceImpl implements AuthService {
    @Resource
    private AuthMapper authMapper;


    @Resource
    private StringRedisTemplate stringRedisTemplate;

    @Resource
    private JavaMailSender mailSender;

    /**
     * 发送邮箱验证码
     *
     * @param email
     * @return {@link String }
     */
    public String sendEmail(@PathVariable String email) throws UnsupportedEncodingException, AddressException {
        String key = "msg_" + email;

        ValueOperations valueOperations = stringRedisTemplate.opsForValue();
        String already_have = (String) valueOperations.get(key);
        if (already_have == null) {
            // 随机生成一个6位数字型的字符串
            String code = "123456";
            // 邮件对象
            SimpleMailMessage message = new SimpleMailMessage();
            message.setSubject("医院小程序邮箱验证码");
            message.setText("尊敬的用户您好!\n\n感谢您使用XX医院。\n\n尊敬的: " + email + "您的校验验证码为: " + code + ",有效期2分钟，请不要把验证码信息泄露给其他人,如非本人请勿操作");
            message.setTo(email);
            // 对方看到的发送人
            message.setFrom(new InternetAddress(MimeUtility.encodeText("医院官方") + "<2774668116@qq.com>").toString());
            //发送邮件
            mailSender.send(message);
            valueOperations.set(key, code, 5L, TimeUnit.MINUTES);
            return code;
        } else {
            return already_have;
        }
    }

    /**
     * 验证码验证是否正确
     * @param email
     * @param code
     * @return {@link Integer }
     */
    public Integer verifyCode(String email, String code) {
        int result = 1;
        ValueOperations valueOperations = stringRedisTemplate.opsForValue();
        String msgKey = "msg_" + email;
        Object value = valueOperations.get(msgKey);
        if(value == null){
            result = -1;
        }else if(!code.equals(value)) {
            result = 0;
        }
        // 如果验证码正确，则删除从redis
        if(result == 1){
            stringRedisTemplate.delete(msgKey);
        }
    /*
      1 验证码正确
      0 验证码错误
      -1 验证码过期
     */
        return result;
    }

    @Override
    public User register(RegisterDTO registerDTO) throws AddressException, UnsupportedEncodingException {
        String code = sendEmail(registerDTO.getEmail());
        Integer result = verifyCode(registerDTO.getEmail(), registerDTO.getCode());
        LambdaQueryWrapper<User> queryWrapper = new LambdaQueryWrapper<>();
        if(result == 1){
            queryWrapper.eq(User::getUsername, registerDTO.getUsername());
            Long l = authMapper.selectCount(queryWrapper);
            if(l == 0){
                // 插入用户到数据库
                authMapper.insertUser(registerDTO);
//                emailServiceImpl.sendEmail(registerDTO.getEmail(), "注册确认", "请点击以下链接进行确认...");

                // 返回注册成功的用户对象（可根据需求返回具体的 User 对象）
//                LambdaQueryWrapper<User> queryWrapperByEmail = new LambdaQueryWrapper<>();
//                queryWrapperByEmail.eq(User::getEmai, registerDTO.getEmail());
                return authMapper.findUserByEmail(registerDTO.getEmail()); // 假设有这个方法来查询用户
            }else{
                throw new RuntimeException("用户存在，请重新输入");
            }
        } else if (result == -1) { // 验证码过期
            throw new RuntimeException("验证码已过期，请重新获取"); // 或者定义自定义异常
        } else { // 验证码错误
            throw new RuntimeException("验证码错误，请检查后重试"); // 或者定义自定义异常
        }
    }

}
